unit DirExpMain;

interface

uses Windows, Classes, SysUtils, Forms, ExptIntf, ToolIntf, Dialogs;

type
  TDirectoryExpert = class(TIExpert)
  public
    constructor Create;
    destructor Destroy; override;
    procedure Execute; override;
    function GetAuthor: string; override;
    function GetComment: string; override;
    function GetGlyph: HICON; override;
    function GetIDString: string; override;
    function GetMenuText: string; override;
    function GetName: string; override;
    function GetPage: string; override;
    function GetState: TExpertState; override;
    function GetStyle: TExpertStyle; override;
  private
    FMainItem: TIMenuItemIntf;
    FProjectItem, FImagesItem, FSourcesItem, FDemosItem: TIMenuItemIntf;
    FRootDir, FImagesDir: string;
    procedure MenuClick(Sender: TIMenuItemIntf);
  end;

procedure Register;

implementation

uses Registry;

// Insert a new menu item relative to the item named TargetName

type
  TInsertAction = (iaBefore, iaAfter, iaChild);

resourcestring
  sCannotFindMenuItem = 'Cannot find menu item: %s';
  sNoMenuParent = 'No menu parent for %s';

function InsertMenuItem(Action: TInsertAction;
  TargetName, Caption, Name, Hint: string;
  ShortCut, Context, GroupIndex: Integer;
  Flags: TIMenuFlags;
  EventHandler: TIMenuClickEvent): TIMenuItemIntf;
var
  TargetItem: TIMenuItemIntf;
  ParentItem: TIMenuItemIntf;
  Index: Integer;
begin
  Result := nil;
  // Get the menu bar interface
  with ToolServices.GetMainMenu do
    try
      //  Look up the target menu item
      TargetItem := FindMenuItem(TargetName);
      if TargetItem = nil then
        raise Exception.CreateFmt(sCannotFindMenuItem, [TargetName]);
      try
        if Action = iaChild then
        begin
          // Create a child of the target item
          ParentItem := TargetItem;
          ParentItem.AddRef; // Avoid a double free
          Index := 0;
        end
        else
        begin
          // To create a sibling of the target, get the targets parent
          ParentItem := TargetItem.GetParent;
          Index := TargetItem.GetIndex;
          if Action = iaAfter then
            Inc(Index);
        end;
        if ParentItem = nil then
          raise Exception.CreateFmt(sNoMenuParent, [TargetName]);
        try
          Result := ParentItem.InsertItem(Index,
                        Caption, Name, Hint, ShortCut, Context,
                        GroupIndex, Flags, EventHandler);
        finally
          ParentItem.Free;
        end;
      finally
        TargetItem.Free;
      end;
    finally
      Free;
    end;
end;

// TDirectoryExpert

constructor TDirectoryExpert.Create;
begin
  inherited Create;
  FMainItem := InsertMenuItem(iaBefore, 'ProjectOptionsItem',
    'Change directory', 'IM_ChangeDirectoryItem',
    'Change current directory',
    0, 0, 0, [mfEnabled, mfVisible], nil);
  FProjectItem := InsertMenuItem(iaChild, 'IM_ChangeDirectoryItem',
    'Project directory', 'IM_ProjectDirectoryItem',
    'Change current directory to project directory',
    0, 0, 0, [mfEnabled, mfVisible], MenuClick);
  FImagesItem := InsertMenuItem(iaAfter, 'IM_ProjectDirectoryItem',
    'Images directory', 'IM_ImagesDirectoryItem',
    'Change current directory to images directory',
    0, 0, 0, [mfEnabled, mfVisible], MenuClick);
  FSourcesItem := InsertMenuItem(iaAfter, 'IM_ImagesDirectoryItem',
    'Sources directory', 'IM_SourcesDirectoryItem',
    'Change current directory to sources directory',
    0, 0, 0, [mfEnabled, mfVisible], MenuClick);
  FDemosItem := InsertMenuItem(iaAfter, 'IM_SourcesDirectoryItem',
    'Demos directory', 'IM_DemosDirectoryItem',
    'Change current directory to demos directory',
    0, 0, 0, [mfEnabled, mfVisible], MenuClick);
  // Read registry keys
  with TRegistry.Create do
  try
    RootKey := HKEY_LOCAL_MACHINE;
    OpenKey('\Software\Borland\Delphi\4.0', False);
    FRootDir := ReadString('RootDir');
    OpenKey('\Software\Borland\Borland Shared\Image Files', False);
    FImagesDir := ReadString('RootDir');
  finally
    Free;
  end;
end;

destructor TDirectoryExpert.Destroy;
begin
  FDemosItem.Free;
  FSourcesItem.Free;
  FImagesItem.Free;
  FProjectItem.Free;
  FMainItem.Free;
  inherited Destroy;
end;

procedure TDirectoryExpert.Execute;
begin
end;

function TDirectoryExpert.GetAuthor: string;
begin
  Result := 'Ian Marteens';
end;

function TDirectoryExpert.GetComment: string;
begin
  Result := '';
end;

function TDirectoryExpert.GetGlyph: HICON;
begin
  Result := 0;
end;

function TDirectoryExpert.GetIDString: string;
begin
  Result := 'Ian Marteens.ChangeDir';
end;

function TDirectoryExpert.GetMenuText: string;
begin
  Result := '';
end;

function TDirectoryExpert.GetName: string;
begin
  Result := 'Change directory';
end;

function TDirectoryExpert.GetPage: string;
begin
  Result := '';
end;

function TDirectoryExpert.GetState: TExpertState;
begin
  Result := [];
end;

function TDirectoryExpert.GetStyle: TExpertStyle;
begin
  Result := esAddIn;
end;

procedure TDirectoryExpert.MenuClick(Sender: TIMenuItemIntf);
begin
  if Sender = FProjectItem then
    try
      ChDir(ExtractFilePath(ToolServices.GetProjectName))
    except
      raise Exception.Create('No active project');
    end
  else if Sender = FImagesItem then
    ChDir(FImagesDir)
  else if Sender = FSourcesItem then
    ChDir(FRootDir + '\source')
  else if Sender = FDemosItem then
    ChDir(FRootDir + '\demos');
end;

procedure Register;
begin
  RegisterLibraryExpert(TDirectoryExpert.Create);
end;

end.
